# AUTOGENERATED! DO NOT EDIT! File to edit: ../nbs/00_helper.ipynb.

# %% ../nbs/00_helper.ipynb 2
from __future__ import annotations
from collections import OrderedDict
from collections.abc import Iterable
import datetime
from datetime import timezone
import errno
import glob
from graphlib import TopologicalSorter
from itertools import product
import os
from os import PathLike
from pathlib import Path
from pathvalidate import sanitize_filename
import platform
import re
from typing import Callable, Optional, Pattern, Sequence, Union

from deprecated import deprecated
from natsort import natsorted

# %% auto 0
__all__ = ['ALPHABET_TO_ALPHABET_GROUP_DICT', 'ALPHABET_OR_GREEK_TO_ALPHABET_DICT', 'CHARACTER_ORDERING_LIST',
           'DECORATING_CHARACTERS', 'NONEFFECTIVE_CHARACTERS', 'TO_REMOVE', 'TO_UNDERSCORE', 'find_regex_in_text',
           'replace_string_by_indices', 'double_asterisk_indices', 'notation_asterisk_indices',
           'definition_asterisk_indices', 'defs_and_notats_separations', 'latex_indices', 'is_number', 'existing_path',
           'file_existence_test', 'path_name_no_ext', 'path_no_ext', 'text_from_file', 'files_of_format_sorted',
           'current_time_formatted_to_minutes', 'containing_string_priority', 'default_str_comparison',
           'natsort_comparison', 'graph_for_topological_sort', 'dict_with_keys_topologically_sorted',
           'alphabet_to_alphabet_group', 'alphabet_or_latex_command_to_alphabet',
           'alphabet_or_latex_command_to_alphabet_group', 'latex_to_path_accepted_string']

# %% ../nbs/00_helper.ipynb 5
def _test_directory() -> Path:
    """Returns the `nbs/_tests` directory of the `trouver` repository.
    
    Assumes that the current working directory is either the root of the
    repository or the `nbs` folder and the `nbs/_tests` folder exists in the
    repository but the root of the repository does not have a folder named
    `_tests`.
    """
    cwd = os.getcwd()
    cwd_name = os.path.basename(cwd)
    if cwd_name == 'nbs':
        return Path(cwd) / '_tests'
    else:
        return Path(cwd) / 'nbs'/ '_tests'

# %% ../nbs/00_helper.ipynb 11
def find_regex_in_text(
        text: str, # Text in which to find regex patter
        pattern: str | Pattern[str] # The regex pattern
        ) -> list[tuple[int]]: # Each tuple is of the form `(a,b)` where `text[a:b]` is the regex match.
    # TODO: rename into regex_indices_in_text
    # TODO: swap parameters.
    """Return ranges in `text` where `pattern` occurs.
    """
    matches = re.finditer(pattern, text)
    return [match.span() for match in matches]

# %% ../nbs/00_helper.ipynb 20
def replace_string_by_indices(
        string: str, # String in which to make replacemenets 
        replace_ranges: Sequence[Union[Sequence[int], int]], # A list of lists/tuples of int's or a single list/tuple of int's. Each 
        replace_with: Sequence[str] | str # The str(s) which will replace the substrings at `replace_ranges` in `string`. `replace_with` must be a str exactly when `replace_ranges` is a Sequence of a single Sequence of int.
        ) -> str:  # The str obtained by replacing the substrings at `replace_range` in `string` by the strs specified by `replace_with`.
    """Replace parts of ``string`` at the specified locations"

    Use this with `find_regex_in_text`.

    **Parameters**

    - ``string`` - `str`
    - ``replace_ranges`` - `Sequence[Sequence[int] | int]`
        - Either a list of lists/tuples of one or two int's. A list/tuple
        ``[a,b]`` or ``(a,b)`` means that ``string[a:b]`` is to be replaced.
        ``[a]`` or ``(a)`` means that ``string[a:]`` is to be replaced. The ranges should
        not overlap and should be arranged in chronological order.
    - ``replace_with`` - `Sequence[str] | str`
        - The str's which will replace the parts represented by 
        ``replace_ranges``. ``replace_ranges`` and ``replace_with`` must be
        both lists or both not lists. If they are lists, they must be of 
        the same length.

    **Returns**

    - str

    """
    if isinstance(replace_with, str):
        replace_ranges = [replace_ranges]
        replace_with = [replace_with]
    if len(replace_ranges) != len(replace_with):
        raise ValueError(
            'The lengths of `replace_ranges` and `replace_with` are different.')
    if len(replace_ranges) == 0:
        return string

    str_parts = _str_parts(string, replace_ranges, replace_with)
    return "".join(str_parts)


def _str_parts(string, replace_ranges, replace_with):
    """Divide `string` into parts divided outside of `replace_ranges`
    and with `replace_with` inserted."""
    str_parts = []
    for i, replace_string in enumerate(replace_ranges):
        replace_string = replace_with[i]
        if i > 0 and len(replace_ranges[i-1]) == 1:
            unreplaced_start_index = len(string)
        elif i > 0 and len(replace_ranges[i-1]) != 1:
            unreplaced_start_index = replace_ranges[i-1][1]
        else:
            unreplaced_start_index = 0
        unreplaced_end_index = replace_ranges[i][0]
        str_parts.append(string[unreplaced_start_index:unreplaced_end_index])
        str_parts.append(replace_string)
    # Add the last (unreplaced) part to str_parts.
    if len(replace_ranges[-1]) == 1:
        unreplaced_start_index = len(string)
    else:
        unreplaced_start_index = replace_ranges[-1][1]
    str_parts.append(string[unreplaced_start_index:])
    return str_parts

# %% ../nbs/00_helper.ipynb 27
def double_asterisk_indices(
        text: str # the str in which to find the indices of double asterisk surrounded text.
        ) -> list[tuple[int]]: # Each tuple is of the form `(start,end)`, where `text[start:end]` is a part in `text` with double asterisks, including the double asterisks.
    # TODO: fix double asterisks in math mode.
    """Return the indices in `str` of text surrounded by double asterisks.
    
    Assumes there no LaTeX math mode string has double asterisks.

    **See Also**
    
    - `notation_asterisk_indices`
    - `definition_asterisk_indices`
    """
    return find_regex_in_text(text, pattern='\*\*[^*]+\*\*')



# %% ../nbs/00_helper.ipynb 29
def notation_asterisk_indices(
        text: str # the str in which to find the indices of notations surrounded by double asterisks.
        ) -> list[tuple[int]]: # Each tuple is of the form `(start,end)`, where `text[start:end]` is a part in `text` with LaTeX math mode text with double asterisks, including the double asterisks.
    """Return the indices of notation text surrounded by double asterisks.
    
    A double-asterisk-surrounded-text is a notation almost always
    when it is purely LaTeX math mode text. 

    Assumes that no LaTeX math mode string has the dollar sign character
    within it.
    """
    return find_regex_in_text(
        text, pattern='\*\*\$\$[^$]+\$\$\*\*|\*\*\$[^$]+\$\*\*')
    # I previous used this, but it was not picking up notation LaTeX str
    # containing asterisks, e.g. `**$\pi^*$**``, `**$\pi_*$**`.`
    return find_regex_in_text(
        text, pattern='\*\*\$\$[^*$]+\$\$\*\*|\*\*\$[^*$]+\$\*\*')


def definition_asterisk_indices(
        text: str # The str in which to find the indices of the definitions surrounded by double asterisks.
        ) -> list[tuple[int]]: # Each tuple is of the form `(start,end)`, where `text[start:end]` is a substring in `text` surrounded by double asterisks, including the double asterisks.
    """Return the indices of definition text surrounded by double asterisks.
    
    A double-asterisk-surrounded-text is a definition almost always
    when it is not purely LaTeX math mode text.
    
    Assumes that no LaTeX math mode string has double asterisks and that no
    LaTeX math mode string has the dollar sign character within it.
    """
    all_double_asterisks = double_asterisk_indices(text)
    notations = notation_asterisk_indices(text)
    return [tuppy for tuppy in all_double_asterisks if tuppy not in notations]

# %% ../nbs/00_helper.ipynb 43
def defs_and_notats_separations(
        text: str 
        )-> list[tuple[int, bool]]:
    """Finds the indices in the text where double asterisks occur and
    categorizes whether each index is for a definition or a notation.
    
    **Parameters**

    - text - str

    **Returns**

    - list[tuple[int, bool]]
        - Each tuple is of the form `(start, end, is_notation)`, where
        `text[start:end]` is the double-asterisk surrounded string,
        including the double asterisks.
    """
    all_double_asterisks = double_asterisk_indices(text)
    notations = notation_asterisk_indices(text)
    return [(start, end, (start, end) in notations)
            for start, end in all_double_asterisks]

# %% ../nbs/00_helper.ipynb 47
def latex_indices(text: str) -> list[tuple[int]]:
    """Returns the indices in the text containing LaTeX str.
    
    This may not work correctly if the text has a LaTeX
    formatting issue or if any LaTeX string has a dollar sign `\$`.
    
    **Parameters**

    - text - str

    **Returns**

    - tuple[int]
        - Each tuple is of the form `(start, end)` where
        `text[start:end]` is a LaTeX string, including any leading trailing
        dollar signs (`$` or `$$`).
    """
    # r'(?<!\\)\$.*(?<!\\)\$|(?<!\\)\$(?<!\\)\$.*(?<!\\)\$(?<!\\)\$'
    return find_regex_in_text(text, r'((?<!\\)\$\$?)[^\$]*\1')
    # return find_regex_in_text(text, '\$\$[^\$]*\$\$|\$[^\$]*\$')

# %% ../nbs/00_helper.ipynb 55
def is_number(
        x: Union[float, int, complex, str]
        ) -> bool:
    """Return `True` if the input `x` represents a number.
    
    This function is different from Python's built-in `is_numeric`
    function, which returns `True` when all characters of a string
    are digits.
    """
    if isinstance(x, (float, int, complex)):
        return True
    #For the case where string is None
    if x is None:
        return False
    if x and x[0] == '-': x = x[1:]
    return x.replace(".", "1", 1).isdigit()

# %% ../nbs/00_helper.ipynb 60
def existing_path(
        path: PathLike,  # A file or directory path. Either absolute or relative to `relative_to`.
        relative_to: Optional[PathLike] = None  # Path to the directory that `file` is relative to.  If `None`, then `path` is an absolute path.
        ) -> Path: # The path formed by `relative_to` adjoined with `path`.  Defaults to `None`
    """Returns a path relative to a specified path as an absolute path
    that exists.

    **Raises**

    - `FileNotFoundError`
        - If `relative_to` is not `None` but does not exist, or if
        `file` does not exist.
    - `ValueError`
        - If `relative_to` is not `None` and yet not an absolute path, or
        if `relative_to` is `None` at yet `path` is not an absolute path.
    
    **Notes**
    - This function may add the string `'\\\\?\\'` in front, which identifies
    very long paths.
    """
    if relative_to is not None:
        if not os.path.isabs(relative_to):
            raise ValueError(
                f'The parameter `relative_to` is expected to be an'
                f' absolute path, but it is not: {relative_to}')
        if not os.path.exists(relative_to):
            raise FileNotFoundError(
                errno.ENOENT, os.strerror(errno.ENOENT), relative_to)
        path = Path(relative_to) / path
    elif not os.path.isabs(path):
        raise ValueError(
            f'The parameter `path` is expected to be an absolute path,'
            f' but it is not: {path}')
    if not os.path.exists(path) and platform.system() == 'Windows':
        path = f'\\\\?\\{str(path)}'  # For long file names
    if not os.path.exists(path):
        raise FileNotFoundError(
            errno.ENOENT, os.strerror(errno.ENOENT), path)
    return Path(path)


@deprecated(reason="The function has been renamed to `existing_path`")
def file_existence_test(
        path: PathLike,  # A file or directory path. Either absolute or relative to `relative_to`.
        relative_to: Optional[PathLike] = None  # Path to the directory that `file` is relative to.  If `None`, then `path` is an absolute path.
        ) -> Path: # The path formed by `relative_to` adjoined with `path`.  Defaults to `None`
    """
    **Deprecated. Use `existing_path` instead.**
    
    Returns a path relative to a specified path as an absolute path
    that exists.

    **Raises**
    - `FileNotFoundError`
        - If `relative_to` is not `None` but does not exist, or if
        `file` does not exist.
    
    **Notes**
    - This function may add the string `'\\\\?\\'` in front, which identifies
    very long paths.
    """
    if relative_to is not None:
        if not os.path.isabs(relative_to):
            raise ValueError(
                f'The parameter `relative_to` is expected to be an'
                f' absolute path, but it is not: {relative_to}')
        if not os.path.exists(relative_to):
            raise FileNotFoundError(
                errno.ENOENT, os.strerror(errno.ENOENT), relative_to)
        path = Path(relative_to) / path
    elif not os.path.isabs(path):
        raise ValueError(
            f'The parmaeter `path` is expected to be an absolute path,'
            f' but it is not: {path}')
    if not os.path.exists(path) and platform.system() == 'Windows':
        path = f'\\\\?\\{str(path)}'  # For long file names
    if not os.path.exists(path):
        raise FileNotFoundError(
            errno.ENOENT, os.strerror(errno.ENOENT), path)
    return Path(path)

# %% ../nbs/00_helper.ipynb 73
def path_name_no_ext(
        path: PathLike # The path of the file or directory. This may be absolute or relative to any directory.
        ) -> str: # The name of the file or directory without the extension.
    """Return the name of a file or directory from its path without the
    extension.
    
    The file or directory does not have to exist.
    """
    name_with_extension = os.path.basename(path)
    return os.path.splitext(name_with_extension)[0]

# %% ../nbs/00_helper.ipynb 80
def path_no_ext(
    path: PathLike # The path of the file or directory. This may be absolute or relative to any directory.
    ) -> str: # The path of the file or directory without the extension. If `path` is a path to a directory, then the output should be essentially the same as `path`.
    """Returns the path of a file or directory without the extension.
    
    The file or directory does not have to exist.
    """
    return os.path.splitext(str(path))[0]

# %% ../nbs/00_helper.ipynb 84
def text_from_file(
        path: PathLike, # The absolute path of the file.
        encoding: str = 'utf8' # The encoding of the file to be read. Defaults to `'utf8'`.
        ) -> str: # The entire text from a file
    """Return the entire text from a file.

    Assuems that the file can be encoded in the specified `encoding`
    """
    with open(path, 'r', encoding=encoding) as file:
        text = file.read()
        file.close()
    return text

# %% ../nbs/00_helper.ipynb 87
def files_of_format_sorted(
        directory: PathLike, # The directory in which to find the files
        extension: str = 'txt' # Extension of the files to find. Defaults to 'txt'.
        ) -> list[str]:
    """Return a list of path str of files in the directory (but not subdirectories)
    sorted via `natsort`.
    """
    return natsorted(glob.glob(str(Path(directory) / f'*.{extension}')))

# %% ../nbs/00_helper.ipynb 91
def current_time_formatted_to_minutes(
        ) -> str:
    """Return the current time to minutes.

    **Returns**

    - str
        - In UTC time, to minutes.
    """
    dt = datetime.datetime.now(timezone.utc)
    formatted = dt.isoformat(timespec='minutes')
    return formatted[:16]

# %% ../nbs/00_helper.ipynb 99
def containing_string_priority(str1: str, str2: str) -> int:
    """Returns 1, 0, -1 depending on whether one string contains the other.
    
    TODO make the string containment criterion looser, e.g. finite Galois etale cover
    "contains" finite etale cover.
    
    **Parameters**
    - str1 - str
    - str2 - str
    """
    if str2 in str1:
        return -1
    elif str1 in str2:
        return 1
    else:
        # return len(str2) - len(str1)
        return 0


def default_str_comparison(str1: str, str2: str) -> int:
    """
    
    **Parameters**
    - str1 - str
    - str2 - str
    """
    if str1 > str2:
        return 1
    elif str1 < str2:
        return -1
    else:
        return 0


def natsort_comparison(str1: str, str2: str) -> int:
    """
    
    **Parameters**
    - str1 - str
    - str2 - str
    """
    if str1 == str2:
        return 0
    listy = [str1, str2]
    natsorted(listy)
    if listy[0] == str1:
        return -1
    else:
        return 1

# %% ../nbs/00_helper.ipynb 100
def graph_for_topological_sort(
        items_to_sort: Iterable[str],
        key_order: Callable[[str, str], int]) -> dict[str, set[str]]:
    """
    **Parameters**
    - items_to_sort - Iterable[str]
    - key_order: Callable[[str, str], int]
        - Comparing str1 against str2 results in a positive number if
        str1 is "greater" than str2 (i.e. str1 is of a later priority)
    
    **Returns**
    - dict[str, set[str]]
        - A dict whose keys are the elements `k` of `items_to_sort` and
        whose values are sets of elements `k2` of `items_to_sort` such that
        `key_order(k, k2) > 0`.
    """
    graph = dict()
    for key_1, key_2 in product(items_to_sort, items_to_sort):
        # print(key_1, key_2)
        if key_1 == key_2:
            continue
        # print(key_1, key_2)
        # print(key_order(key_1, key_2))
        if key_1 not in graph:
            graph[key_1] = set()
        if key_order(key_1, key_2) > 0:
            graph[key_1].add(key_2)
    return graph

# %% ../nbs/00_helper.ipynb 101
def dict_with_keys_topologically_sorted(
        dict_to_sort: dict[str],
        key_order: Callable[[str, str], int],
        reverse: bool = False) -> OrderedDict[str]:
    """Returns an OrderedDict whose keys are sorted topologically by
    the specified key order.
    
    **Parameters**
    - dict_to_sort - dict[str]
        - The dict whose keys need to be ordered.
    - key_order
        - The comparison function on the keys of `dict_to_sort`. Defaults to
        the key function for the comparison ``containing_string_priority``.
    - reverse - bool
        - Defaults to `False`
        
    **Returns**
    - OrderedDict[str]
    """
    graph = graph_for_topological_sort(dict_to_sort, key_order)
    ts = TopologicalSorter(graph)
    keys_ordered = list(ts.static_order())
    if reverse:
        keys_ordered = list(reversed(keys_ordered))
    return OrderedDict((key, dict_to_sort[key]) for key in keys_ordered)


# %% ../nbs/00_helper.ipynb 104
ALPHABET_TO_ALPHABET_GROUP_DICT = {'A': 'A-E', 'B': 'A-E', 'C': 'A-E', 'D': 'A-E', 'E': 'A-E', 'F': 'F-J', 'G': 'F-J', 'H': 'F-J', 'I': 'F-J', 'J': 'F-J', 'K': 'K-O', 'L': 'K-O', 'M': 'K-O', 'N': 'K-O', 'O': 'K-O', 'P': 'P-T', 'Q': 'P-T', 'R': 'P-T', 'S': 'P-T', 'T': 'P-T', 'U': 'U-Z', 'V': 'U-Z', 'W': 'U-Z', 'X': 'U-Z', 'Y': 'U-Z', 'Z': 'U-Z'}
ALPHABET_OR_GREEK_TO_ALPHABET_DICT = {}
def alphabet_to_alphabet_group(character) -> str:
    """
    Returns the alphabet group
    
    In my vaults, I often alphabetize things and also group
    the alphabet as follows:
    - A-E
    - F-J
    - K-O
    - P-T
    - U-V
    
    **Parameters**
    - character - str
    
    **Returns**
    - str or `None`
        - Returns `None` if `character` is not an alphabet.
    """
    character = character.upper()
    if character in ALPHABET_TO_ALPHABET_GROUP_DICT:
        return ALPHABET_TO_ALPHABET_GROUP_DICT[character]
    else:
        return None

def alphabet_or_latex_command_to_alphabet(character):
    """Returns the alphabet that the character "corresponds to".
    
    """
    # TODO
    return

def alphabet_or_latex_command_to_alphabet_group(character):
    return alphabet_to_alphabet_group(
        alphabet_or_latex_command_to_alphabet(character))

# %% ../nbs/00_helper.ipynb 107
CHARACTER_ORDERING_LIST =\
    ['A', 'a', r'\Alpha', r'\alpha', 'B', 'b', r'\Beta', r'\beta', 'C', 'c', r'\Gamma',
     r'\gamma', 'D', 'd', r'\Delta', r'\delta', 'E', 'e', r'\Epsilon', r'\epsilon',
     'F', 'f', 'G', 'g', 'H', 'h', r'\Eta', r'\eta', 'I', 'i', r'\Iota', r'\iota',
     'J', 'j', 'K', 'k', r'\Kappa', r'\kappa', 'L', 'l', r'\Lambda', r'\lambda', 'M',
     'm', r'\Mu', r'\mu', 'N', 'n', r'\Nu', r'\nu', 'O', 'o', r'\Omicron', r'\omicron'
     'P', 'p', r'\Pi', r'\pi', r'\Phi', r'\phi', r'\Psi', r'\psi', 'Q', 'q', 'R', 'r', 
     r'\Rho', r'\rho', 'S', 's', r'\Sigma', r'\sigma', 'T', 't', r'\Theta', r'\theta',
     r'\Tau', r'\tau', 'U', 'u', r'\Upsilon', r'\upsilon', 'V', 'v', 'W', 'w', r'\Omega', r'\omega',
     'X', 'x', '\Chi', '\chi', 'Y', 'y', 'Z', 'z', '\Zeta', '\zeta', '*', r'\bullet']
DECORATING_CHARACTERS =\
    [r'\tilde', r'\hat', r'\overline', r'\bar', r'\mathscr', r'\mathcal',
     r'\mathfrak', r'\\operatorname', r'\\text', r'\\bf']
NONEFFECTIVE_CHARACTERS =\
    ['^', '_', '{', '}', '(', ')', '[', ']']

# %% ../nbs/00_helper.ipynb 108
TO_REMOVE = [
    '.', '\'', '$', ')', '{', '}', ':', '?', '!', '#', '%', '&',
    '\\', '<', '>', '*', '?', '"', '@', '+', '`', '|', '=', '[', ']',
    'mathscr', 'mathbf', 'mathrm', 'mathfrak', 'mathcal', 'mathbb', 'operatorname',
    'left', 'right']
TO_UNDERSCORE = [' ', '-', '^', '(', ',', '/']

# TODO: make a universal latex to path string; it seems that latex.convert
# might do something different when naming files.

def latex_to_path_accepted_string(latex: str) -> str:
    """Convert a latex string to a path accepted string
    """
    for to_remove in TO_REMOVE:
        latex, _ = re.subn(re.escape(to_remove), '', latex)
    for to_underscore in TO_UNDERSCORE:
        latex, _ = re.subn(re.escape(to_underscore), '_', latex)
    latex = sanitize_filename(latex)
    return latex



# AUTOGENERATED! DO NOT EDIT! File to edit: ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb.

# %% auto 0
__all__ = ['LABEL_TAGS', 'note_is_labeled_with_tag', 'note_labels', 'gather_information_note_types',
           'append_to_information_note_type_database', 'predict_text_types', 'predict_note_types',
           'automatically_add_note_type_tags', 'convert_auto_tags_to_regular_tags_in_notes']

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 3
LABEL_TAGS = [
    '#_meta/concept', '#_meta/exercise', '#_meta/definition', '#_meta/example', 
    '#_meta/narrative', '#_meta/notation', '#_meta/proof', '#_meta/remark',
    '#_meta/TODO/split', '#_meta/TODO/merge', '#_meta/TODO/delete', '#_meta/hint',
    '#_meta/how_to', '#_meta/conjecture', '#_meta/convention',
    '#_meta/context',
]

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 6
import os
from os import PathLike
from pathlib import Path
import shutil
from typing import Optional
import warnings

from fastai.text.learner import TextLearner
import pandas as pd

from .....helper import current_time_formatted_to_minutes
from ....markdown.file import MarkdownFile
from .database_update import max_ID, append_to_database
from ..note_processing import process_standard_information_note
from ...vault import VaultNote

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 9
def note_is_labeled_with_tag(
        note: VaultNote,
        label_tag: str # A tag which labels a type that `note` is. Includes the beginning hashtag `#`, e.g. `#_meta/definition`, `#_meta/TODO/split`
        ) -> bool: # `True` if `note` is labeled as type `label_type`.
    """
    Return `True` if the standard information note is labeled as
    begin a specified type.

    **Raises**

    - `ValueError`
        - If `label_tag` does not include the beginning hashtag `#`.
    """
    # assert is_standard_information_note
    if not label_tag.startswith('#'):
        raise ValueError(f"`label_tag` does not start with a hashtag `#`: {label_tag}")
    label_tag = label_tag[1:]
    mf = MarkdownFile.from_vault_note(note)
    return label_tag in mf.metadata()['tags']



# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 11
def note_labels(
        note: VaultNote
        ) -> dict[str, str]:
        # Each key is a string, which is a tag, including the starting hashtag `#`. Each value is a string, either `'IS {tag}'` or `'NOT {tag}'`.
    """Return a dict indicating what labels a note has.

    The labels come from the `LABEL_TAGS` dict.
    """
    label_dict = {label_tag: note_is_labeled_with_tag(note, label_tag)
                  for label_tag in LABEL_TAGS}
    return {tag: (f'IS {tag}' if flag else f'NOT {tag}')
            for tag, flag in label_dict.items()}
    

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 14
def gather_information_note_types(
        vault: PathLike,
        notes: list[VaultNote],
        ) -> pd.DataFrame: # Has columns `Time added`, `Time modified`, `Note name`, `Full note content`, `Processed note content` as well as columns for each tag label. See `append_to_information_note_type_database` for more details about these columns.
    """
    Return a `pandas.DataFrame` encapsulating the data of note labels.
    """
    labels_of_notes = [note_labels(note) for note in notes]
    rows = []
    current_time = current_time_formatted_to_minutes()
    for i, (note, labels_of_note) in enumerate(zip(notes, labels_of_notes)):
        mf = MarkdownFile.from_vault_note(note)
        rows.append({
            'Time added': current_time,
            'Time modified': current_time,
            'Note name': note.name,
            'Full note content': str(mf), 
            'Processed note content': str(process_standard_information_note(
                mf, vault)),
            **labels_of_note
        })
    return pd.DataFrame(rows)
    # notes_with_processed_text_and_
    # process_standard_information_note


# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 16
def append_to_information_note_type_database(
        vault: PathLike, # The vault freom which the data is drawn
        file: PathLike, # The path to a CSV file
        notes: list[VaultNote], # the notes to add to the database
        backup: bool = True # If `True`, makes a copy of `file` in the same directory and with the same name, except with an added extension of `.bak`.
        ) -> None:
    """
    Either create a `csv` file containing data for information note type
    labels or append to an existing `csv` file.

    The columns of the database file are as follows:

    - `Time added` - The time when the row was added.
    - `Time modified` - The time when the labels of the row 
    - `Note name` - The name of the note from which the data for the row
      was derived.
    - `Full note content` - The entire content/text of the note.
    - `Processed note content` - The "raw" content of the note without
      the YAML frontmatter meta, Markdown headings, links, footnotes, etc. 

    All timestamps are in UTC time and specify time to minutes
    (i.e. no seconds/microseconds).
    
    If a "new" note has the same processed content as a pre-existing
    note and anything is different about the "new" note, then update
    the row of the existing note. In particular, the following are updated:
    - Time modified (set to current time)
    - Note name (overwritten)
    - Full note content (overwritten)
    - Columns for categorization (overwritten)
    
    This method assumes that all the processed content in the
    CSV file are all distinct if the CSV file exists.
    """
    if not notes:
      return
    file = Path(file)
    df = pd.read_csv(file) if os.path.exists(file) else None
    # start_ID_from = max_ID(df) + 1 if not df is None else 1
    new_df = gather_information_note_types(vault, notes)
    cols = [
        'Time added', 'Time modified', 'Note name',
        'Full note content', 'Processed note content']
    cols.extend(LABEL_TAGS)
    cols_to_update = ['Time modified', 'Note name', 'Full note content']
    cols_to_update.extend(LABEL_TAGS)
    append_to_database(
        file, new_df, cols, 'Processed note content', cols_to_update, backup)


# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 20
def predict_text_types(
        learn: TextLearner, # The ML model predicting note types.
        texts: list[str],
        remove_NO_TAG: bool = True # If `True`, remove `NO_TAG`, which in theory is supposed to indicate that no types are predicted, but in practice can somehow be predicted along with actual types.
        ) -> list[list[str]]: # Each list corresponds to a text and contains the predicted types of the text.
    """Predict the types of mathematical texts using an ML model."""
    predictions = []
    for text in texts:
        with learn.no_bar(), learn.no_logging():
            pred, loss, _ = learn.predict(text)
        if remove_NO_TAG and 'NO_TAG' in pred:
            pred.remove('NO_TAG')
        predictions.append(list(pred))
    return predictions

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 24
def predict_note_types(
        learn: TextLearner, # The ML model predicting note types.
        vault: PathLike, # The vault with the notes.
        notes: list[VaultNote], # The notes with texts to predict
        remove_NO_TAG: bool = True # If `True`, remove `NO_TAG`, which in theory is supposed to indicate that no types are predicted, but in practice can somehow be predicted along with actual types.
        ) -> list[list[str]]:
    markdown_files = [
        MarkdownFile.from_vault_note(note) for note in notes]
    raw_note_contents = [
        str(process_standard_information_note(mf, vault)) for mf in markdown_files]
    return predict_text_types(learn, raw_note_contents, remove_NO_TAG)

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 28
def automatically_add_note_type_tags(
        learn: TextLearner, # The ML model predicting note types.
        vault: PathLike, # The vault with the notes
        notes: list[VaultNote],
        add_auto_label: bool = True, # If `True`, adds `"_auto"` to the front of the note type tag to indicate that the tags were added via this automated script.
        overwrite: Optional[str] = None # Either `'w'`, `'a'`, or `None`. If `'w'`, then overwrite any already-existing note type tags (from LABEL_TAGS) with the predicted tags. If `'a'`, then preserve already-existing note type tags and just append the newly predicted ones; in the case that `learn` predicts the note type whose tag is already in the note, a new tag of that type is not added, even if `add_auto_label=True`. If `None`, then do not make modifications to each note if any note type tags already exist in the note; if the predicted note types are different from the already existing note types, then raise a warning.
        ) -> None:
    """
    Predict note types and add the predicted types as
    frontmatter YAML tags in the notes.

    Non-`_auto`-labeled tags take precedent over `auto`-labeled tags,
    unless `overwrite='w'`.
    
    **Raises**

    - Warning:
        - If `overwrite=None`, a note already has some note type tags,
        and `learn` predicts different note types as those in the note.
    
    """
    if overwrite not in ['w', 'a'] and overwrite is not None:
        raise ValueError(
            f"`overwrite` was expected to be 'w', 'a', or None," 
            f" but was {overwrite}")
    predictions = predict_note_types(learn, vault, notes)
    # remove hashtags
    predictions = [[tag[1:] if tag.startswith('#') else tag for tag in tags]
                   for tags in predictions]
    # Add `_auto/`
    all_label_tags = [*LABEL_TAGS]
    all_label_tags.extend([f'_auto/{tag}' for tag in LABEL_TAGS])
    for note, prediction in zip(notes, predictions):
        _change_label_tags_for_single_note(
            note, prediction, overwrite, add_auto_label,
            all_label_tags)


def _change_label_tags_for_single_note(
        note: VaultNote, prediction: list[str], overwrite: Optional[str],
        add_auto_label: bool, all_label_tags: list[str]):
    mf = MarkdownFile.from_vault_note(note)

    if add_auto_label:
        tags_to_add = _auto_prediction(prediction)
    else:
        tags_to_add = prediction

    if overwrite == 'w':
        mf.remove_tags(all_label_tags)
        mf.add_tags(tags_to_add, skip_repeated_auto=True)
    elif overwrite == 'a':
        for tag in prediction:
            _append_single_predicted_tag(mf, tag, add_auto_label)
    else:  # overwrite=None
        # TODO
        if not _has_any_label_tags(mf):
            mf.add_tags(tags_to_add, skip_repeated_auto=True)
        elif not _has_exactly_predicted_tags(mf, prediction):
            warnings.warn(
                "The note type labeling tags in the note are different"
                f" from the predicted note types:\n\nNote name: {note.name}"
                f"\n\nPredicted types: {prediction}", UserWarning)        
    mf.write(note)


def _auto_prediction(prediction: list[str]):
    return [f'_auto/{tag}' for tag in prediction]


def _append_single_predicted_tag(
        mf, tag, add_auto_label):
    if tag in mf.metadata()['tags']:
        return
    elif f'_auto/{tag}' in mf.metadata()['tags'] and add_auto_label:
        return
    elif f'_auto/{tag}' in mf.metadata()['tags'] and not add_auto_label:
        mf.remove_tags([f'_auto/{tag}'])
        mf.add_tags([tag])
    else:
        mf.add_tags([f'_auto/{tag}'] if add_auto_label else [tag])
    

def _has_exactly_predicted_tags(
        mf, prediction: list[str]) -> bool:
    """Return `True` if the MarkdownFile already has the predicted tags
    (or the corresponding `_auto` tags)"""
    for tag in LABEL_TAGS:
        if (mf.has_tag(tag) or mf.has_tag(f'_auto/{tag}')) and tag in prediction:
            continue
        else:
            return False
    return True


def _has_any_label_tags(
        mf) -> bool:
    """Return `True` if the MarkdownFile has any label tags (or correspnoding `_auto` tags)"""
    for tag in LABEL_TAGS:
        if mf.has_tag(tag) or mf.has_tag(f'_auto/{tag}'):
            continue
        else:
            return False
    return True

# %% ../../../../../nbs/23_markdown.obsidian.personal.machine_learning.information_note_types.ipynb 39
def convert_auto_tags_to_regular_tags_in_notes(
        notes: list[VaultNote], 
        exclude: list[str] = ['links_added', 'notations_added'] # The tags whose `_auto/` tags should not be converted. The str should not start with `'#'` and should not start with `'_auto/'`.
        ) -> None:
    """Convert the auto tags into regular tags for the notes.
    """
    for note in notes:
        mf = MarkdownFile.from_vault_note(note)
        mf.replace_auto_tags_with_regular_tags(exclude)
        mf.write(note)



# Autogenerated by nbdev

d = { 'settings': { 'branch': 'main',
                'doc_baseurl': '/trouver',
                'doc_host': 'https://hyunjongkimmath.github.io',
                'git_url': 'https://github.com/hyunjongkimmath/trouver',
                'lib_path': 'trouver'},
  'syms': { 'trouver.core': {'trouver.core.foo': ('core.html#foo', 'trouver/core.py')},
            'trouver.helper': { 'trouver.helper._str_parts': ('helper.html#_str_parts', 'trouver/helper.py'),
                                'trouver.helper._test_directory': ('helper.html#_test_directory', 'trouver/helper.py'),
                                'trouver.helper.alphabet_or_latex_command_to_alphabet': ( 'helper.html#alphabet_or_latex_command_to_alphabet',
                                                                                          'trouver/helper.py'),
                                'trouver.helper.alphabet_or_latex_command_to_alphabet_group': ( 'helper.html#alphabet_or_latex_command_to_alphabet_group',
                                                                                                'trouver/helper.py'),
                                'trouver.helper.alphabet_to_alphabet_group': ( 'helper.html#alphabet_to_alphabet_group',
                                                                               'trouver/helper.py'),
                                'trouver.helper.containing_string_priority': ( 'helper.html#containing_string_priority',
                                                                               'trouver/helper.py'),
                                'trouver.helper.current_time_formatted_to_minutes': ( 'helper.html#current_time_formatted_to_minutes',
                                                                                      'trouver/helper.py'),
                                'trouver.helper.default_str_comparison': ('helper.html#default_str_comparison', 'trouver/helper.py'),
                                'trouver.helper.definition_asterisk_indices': ( 'helper.html#definition_asterisk_indices',
                                                                                'trouver/helper.py'),
                                'trouver.helper.defs_and_notats_separations': ( 'helper.html#defs_and_notats_separations',
                                                                                'trouver/helper.py'),
                                'trouver.helper.dict_with_keys_topologically_sorted': ( 'helper.html#dict_with_keys_topologically_sorted',
                                                                                        'trouver/helper.py'),
                                'trouver.helper.double_asterisk_indices': ('helper.html#double_asterisk_indices', 'trouver/helper.py'),
                                'trouver.helper.existing_path': ('helper.html#existing_path', 'trouver/helper.py'),
                                'trouver.helper.file_existence_test': ('helper.html#file_existence_test', 'trouver/helper.py'),
                                'trouver.helper.files_of_format_sorted': ('helper.html#files_of_format_sorted', 'trouver/helper.py'),
                                'trouver.helper.find_regex_in_text': ('helper.html#find_regex_in_text', 'trouver/helper.py'),
                                'trouver.helper.graph_for_topological_sort': ( 'helper.html#graph_for_topological_sort',
                                                                               'trouver/helper.py'),
                                'trouver.helper.is_number': ('helper.html#is_number', 'trouver/helper.py'),
                                'trouver.helper.latex_indices': ('helper.html#latex_indices', 'trouver/helper.py'),
                                'trouver.helper.latex_to_path_accepted_string': ( 'helper.html#latex_to_path_accepted_string',
                                                                                  'trouver/helper.py'),
                                'trouver.helper.natsort_comparison': ('helper.html#natsort_comparison', 'trouver/helper.py'),
                                'trouver.helper.notation_asterisk_indices': ('helper.html#notation_asterisk_indices', 'trouver/helper.py'),
                                'trouver.helper.path_name_no_ext': ('helper.html#path_name_no_ext', 'trouver/helper.py'),
                                'trouver.helper.path_no_ext': ('helper.html#path_no_ext', 'trouver/helper.py'),
                                'trouver.helper.replace_string_by_indices': ('helper.html#replace_string_by_indices', 'trouver/helper.py'),
                                'trouver.helper.text_from_file': ('helper.html#text_from_file', 'trouver/helper.py')},
            'trouver.latex.convert': { 'trouver.latex.convert.NoDocumentNodeError': ( 'latex.convert.html#nodocumentnodeerror',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert.NoDocumentNodeError.__init__': ( 'latex.convert.html#nodocumentnodeerror.__init__',
                                                                                               'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjust_common_section_titles_in_parts': ( 'latex.convert.html#_adjust_common_section_titles_in_parts',
                                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjust_common_syntax_to_markdown_in_parts': ( 'latex.convert.html#_adjust_common_syntax_to_markdown_in_parts',
                                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._adjusted_title': ( 'latex.convert.html#_adjusted_title',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._append_non_environment_accumulation_to_parts_if_non_empty': ( 'latex.convert.html#_append_non_environment_accumulation_to_parts_if_non_empty',
                                                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._argument_detection': ( 'latex.convert.html#_argument_detection',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert._change_counters': ( 'latex.convert.html#_change_counters',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._commands_from_newcommand_and_declaremathoperator': ( 'latex.convert.html#_commands_from_newcommand_and_declaremathoperator',
                                                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._consider_part_to_add': ( 'latex.convert.html#_consider_part_to_add',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_note_for_part': ( 'latex.convert.html#_create_note_for_part',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_notes_from_parts': ( 'latex.convert.html#_create_notes_from_parts',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._create_part_or_update': ( 'latex.convert.html#_create_part_or_update',
                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_environment_node': ( 'latex.convert.html#_is_environment_node',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_numbered': ( 'latex.convert.html#_is_numbered',
                                                                               'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_numberedwithin': ( 'latex.convert.html#_is_numberedwithin',
                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_section_node': ( 'latex.convert.html#_is_section_node',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._is_subsection_node': ( 'latex.convert.html#_is_subsection_node',
                                                                                      'trouver/latex/convert.py'),
                                       'trouver.latex.convert._make_links_in_index_note_for_section': ( 'latex.convert.html#_make_links_in_index_note_for_section',
                                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert._make_links_in_index_notes': ( 'latex.convert.html#_make_links_in_index_notes',
                                                                                             'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_is_proof_immediately_following_a_theorem_like_environment': ( 'latex.convert.html#_node_is_proof_immediately_following_a_theorem_like_environment',
                                                                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_numbering': ( 'latex.convert.html#_node_numbering',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._node_warrants_own_part': ( 'latex.convert.html#_node_warrants_own_part',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert._numbering_helper': ( 'latex.convert.html#_numbering_helper',
                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._part_starts_section': ( 'latex.convert.html#_part_starts_section',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._part_starts_subsection': ( 'latex.convert.html#_part_starts_subsection',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert._process_node': ( 'latex.convert.html#_process_node',
                                                                                'trouver/latex/convert.py'),
                                       'trouver.latex.convert._replace_command': ( 'latex.convert.html#_replace_command',
                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._replace_custom_commands_in_parts': ( 'latex.convert.html#_replace_custom_commands_in_parts',
                                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert._search_counters_by_pattern': ( 'latex.convert.html#_search_counters_by_pattern',
                                                                                              'trouver/latex/convert.py'),
                                       'trouver.latex.convert._search_display_names_by_pattern': ( 'latex.convert.html#_search_display_names_by_pattern',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert._section_title': ( 'latex.convert.html#_section_title',
                                                                                 'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_all_numberwithins': ( 'latex.convert.html#_setup_all_numberwithins',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_counters': ( 'latex.convert.html#_setup_counters',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert._setup_numberwithins': ( 'latex.convert.html#_setup_numberwithins',
                                                                                       'trouver/latex/convert.py'),
                                       'trouver.latex.convert._title': ('latex.convert.html#_title', 'trouver/latex/convert.py'),
                                       'trouver.latex.convert._title_for_environment_node': ( 'latex.convert.html#_title_for_environment_node',
                                                                                              'trouver/latex/convert.py'),
                                       'trouver.latex.convert._unnumbered_environments': ( 'latex.convert.html#_unnumbered_environments',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert._update_links_to_make': ( 'latex.convert.html#_update_links_to_make',
                                                                                        'trouver/latex/convert.py'),
                                       'trouver.latex.convert.adjust_common_syntax_to_markdown': ( 'latex.convert.html#adjust_common_syntax_to_markdown',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert.custom_commands': ( 'latex.convert.html#custom_commands',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.display_names_of_environments': ( 'latex.convert.html#display_names_of_environments',
                                                                                                'trouver/latex/convert.py'),
                                       'trouver.latex.convert.divide_latex_text': ( 'latex.convert.html#divide_latex_text',
                                                                                    'trouver/latex/convert.py'),
                                       'trouver.latex.convert.divide_preamble': ( 'latex.convert.html#divide_preamble',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.environment_names_used': ( 'latex.convert.html#environment_names_used',
                                                                                         'trouver/latex/convert.py'),
                                       'trouver.latex.convert.find_document_node': ( 'latex.convert.html#find_document_node',
                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert.get_node_from_simple_text': ( 'latex.convert.html#get_node_from_simple_text',
                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.numbered_newtheorems_counters_in_preamble': ( 'latex.convert.html#numbered_newtheorems_counters_in_preamble',
                                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.numberwithins_in_preamble': ( 'latex.convert.html#numberwithins_in_preamble',
                                                                                            'trouver/latex/convert.py'),
                                       'trouver.latex.convert.regex_pattern_detecting_command': ( 'latex.convert.html#regex_pattern_detecting_command',
                                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.remove_comments': ( 'latex.convert.html#remove_comments',
                                                                                  'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_command_in_text': ( 'latex.convert.html#replace_command_in_text',
                                                                                          'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_commands_in_latex_document': ( 'latex.convert.html#replace_commands_in_latex_document',
                                                                                                     'trouver/latex/convert.py'),
                                       'trouver.latex.convert.replace_commands_in_text': ( 'latex.convert.html#replace_commands_in_text',
                                                                                           'trouver/latex/convert.py'),
                                       'trouver.latex.convert.section_and_subsection_titles_from_latex_parts': ( 'latex.convert.html#section_and_subsection_titles_from_latex_parts',
                                                                                                                 'trouver/latex/convert.py'),
                                       'trouver.latex.convert.setup_reference_from_latex_parts': ( 'latex.convert.html#setup_reference_from_latex_parts',
                                                                                                   'trouver/latex/convert.py'),
                                       'trouver.latex.convert.swap_numbers_invoked': ( 'latex.convert.html#swap_numbers_invoked',
                                                                                       'trouver/latex/convert.py')},
            'trouver.markdown.markdown.file': { 'trouver.markdown.markdown.file.MarkdownFile': ( 'markdown.markdown.file.html#markdownfile',
                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.__init__': ( 'markdown.markdown.file.html#markdownfile.__init__',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.__str__': ( 'markdown.markdown.file.html#markdownfile.__str__',
                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._include_previous_line_as_id_text': ( 'markdown.markdown.file.html#markdownfile._include_previous_line_as_id_text',
                                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._line_dict': ( 'markdown.markdown.file.html#markdownfile._line_dict',
                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._look_at_start_of_file': ( 'markdown.markdown.file.html#markdownfile._look_at_start_of_file',
                                                                                                                        'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._merge_latex_into_text': ( 'markdown.markdown.file.html#markdownfile._merge_latex_into_text',
                                                                                                                        'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._merge_one_display_math_mode_latex_chunk': ( 'markdown.markdown.file.html#markdownfile._merge_one_display_math_mode_latex_chunk',
                                                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._replace_embedded_links_one_line': ( 'markdown.markdown.file.html#markdownfile._replace_embedded_links_one_line',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_embedded_link_of_id': ( 'markdown.markdown.file.html#markdownfile._text_of_embedded_link_of_id',
                                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_embedded_link_of_section': ( 'markdown.markdown.file.html#markdownfile._text_of_embedded_link_of_section',
                                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile._text_of_lines_of_embedded_links': ( 'markdown.markdown.file.html#markdownfile._text_of_lines_of_embedded_links',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_blank_line_to_end': ( 'markdown.markdown.file.html#markdownfile.add_blank_line_to_end',
                                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_line_in_section': ( 'markdown.markdown.file.html#markdownfile.add_line_in_section',
                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_line_to_end': ( 'markdown.markdown.file.html#markdownfile.add_line_to_end',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_metadata_section': ( 'markdown.markdown.file.html#markdownfile.add_metadata_section',
                                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.add_tags': ( 'markdown.markdown.file.html#markdownfile.add_tags',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.clear_all_sections': ( 'markdown.markdown.file.html#markdownfile.clear_all_sections',
                                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.clear_section': ( 'markdown.markdown.file.html#markdownfile.clear_section',
                                                                                                               'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.copy': ( 'markdown.markdown.file.html#markdownfile.copy',
                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_file': ( 'markdown.markdown.file.html#markdownfile.from_file',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_list': ( 'markdown.markdown.file.html#markdownfile.from_list',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_string': ( 'markdown.markdown.file.html#markdownfile.from_string',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.from_vault_note': ( 'markdown.markdown.file.html#markdownfile.from_vault_note',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings': ( 'markdown.markdown.file.html#markdownfile.get_headings',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_and_text': ( 'markdown.markdown.file.html#markdownfile.get_headings_and_text',
                                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_by_line_number': ( 'markdown.markdown.file.html#markdownfile.get_headings_by_line_number',
                                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_headings_tree': ( 'markdown.markdown.file.html#markdownfile.get_headings_tree',
                                                                                                                   'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_line_number_of_heading': ( 'markdown.markdown.file.html#markdownfile.get_line_number_of_heading',
                                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.get_line_numbers_under_heading': ( 'markdown.markdown.file.html#markdownfile.get_line_numbers_under_heading',
                                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.has_metadata': ( 'markdown.markdown.file.html#markdownfile.has_metadata',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.has_tag': ( 'markdown.markdown.file.html#markdownfile.has_tag',
                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.insert_line': ( 'markdown.markdown.file.html#markdownfile.insert_line',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.merge_display_math_mode': ( 'markdown.markdown.file.html#markdownfile.merge_display_math_mode',
                                                                                                                         'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.merge_display_math_mode_into_preceding_text': ( 'markdown.markdown.file.html#markdownfile.merge_display_math_mode_into_preceding_text',
                                                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.metadata': ( 'markdown.markdown.file.html#markdownfile.metadata',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.metadata_lines': ( 'markdown.markdown.file.html#markdownfile.metadata_lines',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.parts_of_id': ( 'markdown.markdown.file.html#markdownfile.parts_of_id',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.pop_line': ( 'markdown.markdown.file.html#markdownfile.pop_line',
                                                                                                          'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_double_blank_lines': ( 'markdown.markdown.file.html#markdownfile.remove_double_blank_lines',
                                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_footnotes_to_embedded_links': ( 'markdown.markdown.file.html#markdownfile.remove_footnotes_to_embedded_links',
                                                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_headers': ( 'markdown.markdown.file.html#markdownfile.remove_headers',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_in_line_tags': ( 'markdown.markdown.file.html#markdownfile.remove_in_line_tags',
                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_line': ( 'markdown.markdown.file.html#markdownfile.remove_line',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_lines': ( 'markdown.markdown.file.html#markdownfile.remove_lines',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_metadata': ( 'markdown.markdown.file.html#markdownfile.remove_metadata',
                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_section': ( 'markdown.markdown.file.html#markdownfile.remove_section',
                                                                                                                'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.remove_tags': ( 'markdown.markdown.file.html#markdownfile.remove_tags',
                                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_auto_tags_with_regular_tags': ( 'markdown.markdown.file.html#markdownfile.replace_auto_tags_with_regular_tags',
                                                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_embedded_links_with_text': ( 'markdown.markdown.file.html#markdownfile.replace_embedded_links_with_text',
                                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_links_with_display_text': ( 'markdown.markdown.file.html#markdownfile.replace_links_with_display_text',
                                                                                                                                 'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.replace_metadata': ( 'markdown.markdown.file.html#markdownfile.replace_metadata',
                                                                                                                  'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.text_of_lines': ( 'markdown.markdown.file.html#markdownfile.text_of_lines',
                                                                                                               'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownFile.write': ( 'markdown.markdown.file.html#markdownfile.write',
                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.MarkdownLineEnum': ( 'markdown.markdown.file.html#markdownlineenum',
                                                                                                     'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._add_line_for_list': ( 'markdown.markdown.file.html#_add_line_for_list',
                                                                                                       'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._enquote': ( 'markdown.markdown.file.html#_enquote',
                                                                                             'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_end_in_line_latex': ( 'markdown.markdown.file.html#_line_end_in_line_latex',
                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_start_and_end_in_line_latex': ( 'markdown.markdown.file.html#_line_start_and_end_in_line_latex',
                                                                                                                      'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._line_start_in_line_latex': ( 'markdown.markdown.file.html#_line_start_in_line_latex',
                                                                                                              'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file._remove_text_id': ( 'markdown.markdown.file.html#_remove_text_id',
                                                                                                    'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.dict_to_metadata_lines': ( 'markdown.markdown.file.html#dict_to_metadata_lines',
                                                                                                           'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.find_front_matter_meta_in_markdown_text': ( 'markdown.markdown.file.html#find_front_matter_meta_in_markdown_text',
                                                                                                                            'trouver/markdown/markdown/file.py'),
                                                'trouver.markdown.markdown.file.replace_embedded_links_with_text': ( 'markdown.markdown.file.html#replace_embedded_links_with_text',
                                                                                                                     'trouver/markdown/markdown/file.py')},
            'trouver.markdown.markdown.heading': { 'trouver.markdown.markdown.heading.heading_level': ( 'markdown.markdown.heading.html#heading_level',
                                                                                                        'trouver/markdown/markdown/heading.py'),
                                                   'trouver.markdown.markdown.heading.heading_title': ( 'markdown.markdown.heading.html#heading_title',
                                                                                                        'trouver/markdown/markdown/heading.py')},
            'trouver.markdown.obsidian.footnotes': { 'trouver.markdown.obsidian.footnotes.embedded_note_of_footnote': ( 'markdown.obsidian.footnotes.html#embedded_note_of_footnote',
                                                                                                                        'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.find_footnote_descriptions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#find_footnote_descriptions_in_markdown_text',
                                                                                                                                          'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.find_footnote_mentions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#find_footnote_mentions_in_markdown_text',
                                                                                                                                      'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.footnote_is_simple_embedded_note': ( 'markdown.obsidian.footnotes.html#footnote_is_simple_embedded_note',
                                                                                                                               'trouver/markdown/obsidian/footnotes.py'),
                                                     'trouver.markdown.obsidian.footnotes.remove_footnote_mentions_in_markdown_text': ( 'markdown.obsidian.footnotes.html#remove_footnote_mentions_in_markdown_text',
                                                                                                                                        'trouver/markdown/obsidian/footnotes.py')},
            'trouver.markdown.obsidian.links': { 'trouver.markdown.obsidian.links.LinkFormatError': ( 'markdown.obsidian.links.html#linkformaterror',
                                                                                                      'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.LinkFormatError.__init__': ( 'markdown.obsidian.links.html#linkformaterror.__init__',
                                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.LinkType': ( 'markdown.obsidian.links.html#linktype',
                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink': ( 'markdown.obsidian.links.html#obsidianlink',
                                                                                                   'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.__init__': ( 'markdown.obsidian.links.html#obsidianlink.__init__',
                                                                                                            'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.__str__': ( 'markdown.obsidian.links.html#obsidianlink.__str__',
                                                                                                           'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink._parse_text_as_wikilink': ( 'markdown.obsidian.links.html#obsidianlink._parse_text_as_wikilink',
                                                                                                                           'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.convert_link_type': ( 'markdown.obsidian.links.html#obsidianlink.convert_link_type',
                                                                                                                     'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.displayed_text': ( 'markdown.obsidian.links.html#obsidianlink.displayed_text',
                                                                                                                  'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.from_text': ( 'markdown.obsidian.links.html#obsidianlink.from_text',
                                                                                                             'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.is_abstract': ( 'markdown.obsidian.links.html#obsidianlink.is_abstract',
                                                                                                               'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.to_regex': ( 'markdown.obsidian.links.html#obsidianlink.to_regex',
                                                                                                            'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.ObsidianLink.to_string': ( 'markdown.obsidian.links.html#obsidianlink.to_string',
                                                                                                             'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links._custom_text_for_links_in_text': ( 'markdown.obsidian.links.html#_custom_text_for_links_in_text',
                                                                                                                     'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links._do_not_remove_link': ( 'markdown.obsidian.links.html#_do_not_remove_link',
                                                                                                          'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.all_custom_text_for_links_in_vault': ( 'markdown.obsidian.links.html#all_custom_text_for_links_in_vault',
                                                                                                                         'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.all_links_in_vault': ( 'markdown.obsidian.links.html#all_links_in_vault',
                                                                                                         'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.find_links_in_markdown_text': ( 'markdown.obsidian.links.html#find_links_in_markdown_text',
                                                                                                                  'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.links_from_text': ( 'markdown.obsidian.links.html#links_from_text',
                                                                                                      'trouver/markdown/obsidian/links.py'),
                                                 'trouver.markdown.obsidian.links.remove_links_from_text': ( 'markdown.obsidian.links.html#remove_links_from_text',
                                                                                                             'trouver/markdown/obsidian/links.py')},
            'trouver.markdown.obsidian.personal.authors': { 'trouver.markdown.obsidian.personal.authors.find_author_file': ( 'markdown.obisidian.personal.authors.html#find_author_file',
                                                                                                                             'trouver/markdown/obsidian/personal/authors.py'),
                                                            'trouver.markdown.obsidian.personal.authors.just_last_name': ( 'markdown.obisidian.personal.authors.html#just_last_name',
                                                                                                                           'trouver/markdown/obsidian/personal/authors.py')},
            'trouver.markdown.obsidian.personal.index_notes': { 'trouver.markdown.obsidian.personal.index_notes._move_notes_under_heading': ( 'markdown.obsidian.personal.index_notes.html#_move_notes_under_heading',
                                                                                                                                              'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.add_link_in_index_note_after_note_link': ( 'markdown.obsidian.personal.index_notes.html#add_link_in_index_note_after_note_link',
                                                                                                                                                           'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.convert_heading_to_folder_name': ( 'markdown.obsidian.personal.index_notes.html#convert_heading_to_folder_name',
                                                                                                                                                   'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.convert_title_to_folder_name': ( 'markdown.obsidian.personal.index_notes.html#convert_title_to_folder_name',
                                                                                                                                                 'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.correspond_headings_with_folder': ( 'markdown.obsidian.personal.index_notes.html#correspond_headings_with_folder',
                                                                                                                                                    'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.get_alphanumeric': ( 'markdown.obsidian.personal.index_notes.html#get_alphanumeric',
                                                                                                                                     'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.get_notes_from_index_note': ( 'markdown.obsidian.personal.index_notes.html#get_notes_from_index_note',
                                                                                                                                              'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.information_notes_linked_in_index_note': ( 'markdown.obsidian.personal.index_notes.html#information_notes_linked_in_index_note',
                                                                                                                                                           'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.make_folders_from_index_note_headers': ( 'markdown.obsidian.personal.index_notes.html#make_folders_from_index_note_headers',
                                                                                                                                                         'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.move_information_notes_to_correct_folder': ( 'markdown.obsidian.personal.index_notes.html#move_information_notes_to_correct_folder',
                                                                                                                                                             'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.move_information_notes_to_correct_folder_for_all_indices': ( 'markdown.obsidian.personal.index_notes.html#move_information_notes_to_correct_folder_for_all_indices',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.subsection_folders': ( 'markdown.obsidian.personal.index_notes.html#subsection_folders',
                                                                                                                                       'trouver/markdown/obsidian/personal/index_notes.py'),
                                                                'trouver.markdown.obsidian.personal.index_notes.subsections_listed_in_index_note': ( 'markdown.obsidian.personal.index_notes.html#subsections_listed_in_index_note',
                                                                                                                                                     'trouver/markdown/obsidian/personal/index_notes.py')},
            'trouver.markdown.obsidian.personal.information_notes': { 'trouver.markdown.obsidian.personal.information_notes._index_note_in_parent_directory': ( 'markdown.obsidian.personal.information_notes.html#_index_note_in_parent_directory',
                                                                                                                                                                'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes._index_note_in_same_directory': ( 'markdown.obsidian.personal.information_notes.html#_index_note_in_same_directory',
                                                                                                                                                              'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes._note_is_not_linked_in_index_note': ( 'markdown.obsidian.personal.information_notes.html#_note_is_not_linked_in_index_note',
                                                                                                                                                                  'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.bulleted_links_of_type_in_section': ( 'markdown.obsidian.personal.information_notes.html#bulleted_links_of_type_in_section',
                                                                                                                                                                  'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.citation_location_string': ( 'markdown.obsidian.personal.information_notes.html#citation_location_string',
                                                                                                                                                         'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.create_generic_info_notes_and_link_to_index': ( 'markdown.obsidian.personal.information_notes.html#create_generic_info_notes_and_link_to_index',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.create_info_notes_and_link_to_index': ( 'markdown.obsidian.personal.information_notes.html#create_info_notes_and_link_to_index',
                                                                                                                                                                    'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.fill_info_note_with_template': ( 'markdown.obsidian.personal.information_notes.html#fill_info_note_with_template',
                                                                                                                                                             'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.index_note_of_a_directory': ( 'markdown.obsidian.personal.information_notes.html#index_note_of_a_directory',
                                                                                                                                                          'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.index_note_of_note': ( 'markdown.obsidian.personal.information_notes.html#index_note_of_note',
                                                                                                                                                   'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.link_info_notes_to_index': ( 'markdown.obsidian.personal.information_notes.html#link_info_notes_to_index',
                                                                                                                                                         'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.links_to_common_terms_notes_in_see_also_section': ( 'markdown.obsidian.personal.information_notes.html#links_to_common_terms_notes_in_see_also_section',
                                                                                                                                                                                'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.main_content': ( 'markdown.obsidian.personal.information_notes.html#main_content',
                                                                                                                                             'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.reference_notes_in_references_section_of_information_note': ( 'markdown.obsidian.personal.information_notes.html#reference_notes_in_references_section_of_information_note',
                                                                                                                                                                                          'trouver/markdown/obsidian/personal/information_notes.py'),
                                                                      'trouver.markdown.obsidian.personal.information_notes.reference_of_information_note': ( 'markdown.obsidian.personal.information_notes.html#reference_of_information_note',
                                                                                                                                                              'trouver/markdown/obsidian/personal/information_notes.py')},
            'trouver.markdown.obsidian.personal.machine_learning.database_update': { 'trouver.markdown.obsidian.personal.machine_learning.database_update.append_to_database': ( 'markdown.obsidian.personal.machine_learninig.database_update.html#append_to_database',
                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/database_update.py'),
                                                                                     'trouver.markdown.obsidian.personal.machine_learning.database_update.max_ID': ( 'markdown.obsidian.personal.machine_learninig.database_update.html#max_id',
                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/database_update.py')},
            'trouver.markdown.obsidian.personal.machine_learning.definition_identification': { 'trouver.markdown.obsidian.personal.machine_learning.definition_identification.definition_identification_data_from_note': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#definition_identification_data_from_note',
                                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py'),
                                                                                               'trouver.markdown.obsidian.personal.machine_learning.definition_identification.definitions_in_text': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#definitions_in_text',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py'),
                                                                                               'trouver.markdown.obsidian.personal.machine_learning.definition_identification.gather_definition_identification_data': ( 'markdown.obsidian.personal.machine_learning.definition_identification.html#gather_definition_identification_data',
                                                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/definition_identification.py')},
            'trouver.markdown.obsidian.personal.machine_learning.information_note_types': { 'trouver.markdown.obsidian.personal.machine_learning.information_note_types._append_single_predicted_tag': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_append_single_predicted_tag',
                                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._auto_prediction': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_auto_prediction',
                                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._change_label_tags_for_single_note': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_change_label_tags_for_single_note',
                                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._has_any_label_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_has_any_label_tags',
                                                                                                                                                                                                'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types._has_exactly_predicted_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#_has_exactly_predicted_tags',
                                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.append_to_information_note_type_database': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#append_to_information_note_type_database',
                                                                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.automatically_add_note_type_tags': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#automatically_add_note_type_tags',
                                                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.convert_auto_tags_to_regular_tags_in_notes': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#convert_auto_tags_to_regular_tags_in_notes',
                                                                                                                                                                                                                       'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.gather_information_note_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#gather_information_note_types',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.note_is_labeled_with_tag': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#note_is_labeled_with_tag',
                                                                                                                                                                                                     'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.note_labels': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#note_labels',
                                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.predict_note_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#predict_note_types',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.information_note_types.predict_text_types': ( 'markdown.obsidian.personal.machine_learning.information_note_types.html#predict_text_types',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/information_note_types.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation': { 'trouver.markdown.obsidian.personal.machine_learning.notation._get_notation_indices_to_add': ( 'markdown.obsidian.personal.machine_learning.notation.html#_get_notation_indices_to_add',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation._notation_data_with_indices': ( 'markdown.obsidian.personal.machine_learning.notation.html#_notation_data_with_indices',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.add_one_double_asts_to_line': ( 'markdown.obsidian.personal.machine_learning.notation.html#add_one_double_asts_to_line',
                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.append_notation_data_to_database': ( 'markdown.obsidian.personal.machine_learning.notation.html#append_notation_data_to_database',
                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.automatically_mark_notations': ( 'markdown.obsidian.personal.machine_learning.notation.html#automatically_mark_notations',
                                                                                                                                                                             'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.notation_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation.html#notation_data_from_note',
                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/notation.py'),
                                                                              'trouver.markdown.obsidian.personal.machine_learning.notation.notation_data_from_text': ( 'markdown.obsidian.personal.machine_learning.notation.html#notation_data_from_text',
                                                                                                                                                                        'trouver/markdown/obsidian/personal/machine_learning/notation.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation_identification': { 'trouver.markdown.obsidian.personal.machine_learning.notation_identification._get_notation_indices_to_add': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#_get_notation_indices_to_add',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification._notation_data_with_indices': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#_notation_data_with_indices',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.add_one_double_asts_to_line': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#add_one_double_asts_to_line',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.append_notation_data_to_database': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#append_notation_data_to_database',
                                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.automatically_mark_notations': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#automatically_mark_notations',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.notation_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#notation_data_from_note',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py'),
                                                                                             'trouver.markdown.obsidian.personal.machine_learning.notation_identification.notation_data_from_text': ( 'markdown.obsidian.personal.machine_learning.notation_identification.html#notation_data_from_text',
                                                                                                                                                                                                      'trouver/markdown/obsidian/personal/machine_learning/notation_identification.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization': { 'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._escape_latex_in_original_in_metadata': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_escape_latex_in_original_in_metadata',
                                                                                                                                                                                                                  'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._get_summary': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_get_summary',
                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._metadata_dict_has_auto_notation_summary_tag': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_metadata_dict_has_auto_notation_summary_tag',
                                                                                                                                                                                                                         'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._notation_has_been_summarized': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_notation_has_been_summarized',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._notation_note_has_auto_summary_tag': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_notation_note_has_auto_summary_tag',
                                                                                                                                                                                                                'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization._write_summary_to_notation_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#_write_summary_to_notation_note',
                                                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_column_for_single_text': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_column_for_single_text',
                                                                                                                                                                                                          'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_summary_to_notation_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_summary_to_notation_note',
                                                                                                                                                                                                            'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.append_to_notation_note_summarization_database': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#append_to_notation_note_summarization_database',
                                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.fix_summary_formatting': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#fix_summary_formatting',
                                                                                                                                                                                                   'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.gather_notation_note_summaries': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#gather_notation_note_summaries',
                                                                                                                                                                                                           'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.get_latex_in_original_from_parsed_notation_note_data': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#get_latex_in_original_from_parsed_notation_note_data',
                                                                                                                                                                                                                                 'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.notation_summarization_data_from_note': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#notation_summarization_data_from_note',
                                                                                                                                                                                                                  'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.single_input_for_notation_summarization': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#single_input_for_notation_summarization',
                                                                                                                                                                                                                    'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py'),
                                                                                            'trouver.markdown.obsidian.personal.machine_learning.notation_summarization.summarize_notation': ( 'markdown.obsidian.personal.machine_learning.notation_summarization.html#summarize_notation',
                                                                                                                                                                                               'trouver/markdown/obsidian/personal/machine_learning/notation_summarization.py')},
            'trouver.markdown.obsidian.personal.machine_learning.notations': {},
            'trouver.markdown.obsidian.personal.notation': { 'trouver.markdown.obsidian.personal.notation._build_replacables_from_groups': ( 'markdown.obsidian.personal.notation.html#_build_replacables_from_groups',
                                                                                                                                             'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._divide_bulleted_list_mf_at_end': ( 'markdown.obsidian.personal.notation.html#_divide_bulleted_list_mf_at_end',
                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._full_notation_string': ( 'markdown.obsidian.personal.notation.html#_full_notation_string',
                                                                                                                                    'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._get_notation_string': ( 'markdown.obsidian.personal.notation.html#_get_notation_string',
                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._latex_in_original_from_notat_notes_to_main_note': ( 'markdown.obsidian.personal.notation.html#_latex_in_original_from_notat_notes_to_main_note',
                                                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._latex_in_original_in_notat': ( 'markdown.obsidian.personal.notation.html#_latex_in_original_in_notat',
                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._look_into_node': ( 'markdown.obsidian.personal.notation.html#_look_into_node',
                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._macro_is_actually_placeholder': ( 'markdown.obsidian.personal.notation.html#_macro_is_actually_placeholder',
                                                                                                                                             'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._main_of_notation_from_text': ( 'markdown.obsidian.personal.notation.html#_main_of_notation_from_text',
                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._make_new_notes_from_sifted_double_asts': ( 'markdown.obsidian.personal.notation.html#_make_new_notes_from_sifted_double_asts',
                                                                                                                                                      'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._part_is_unordered_list_and_is_of_markdownstyle_link': ( 'markdown.obsidian.personal.notation.html#_part_is_unordered_list_and_is_of_markdownstyle_link',
                                                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._raw_notation': ( 'markdown.obsidian.personal.notation.html#_raw_notation',
                                                                                                                            'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation._remove_the_notation_str_and_denotes_in_main_mf': ( 'markdown.obsidian.personal.notation.html#_remove_the_notation_str_and_denotes_in_main_mf',
                                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.add_missing_notation_links_to_information_notes': ( 'markdown.obsidian.personal.notation.html#add_missing_notation_links_to_information_notes',
                                                                                                                                                              'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.add_notation_note_to_see_also': ( 'markdown.obsidian.personal.notation.html#add_notation_note_to_see_also',
                                                                                                                                            'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.index_notation_note_formatted_entry': ( 'markdown.obsidian.personal.notation.html#index_notation_note_formatted_entry',
                                                                                                                                                  'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.main_of_notation': ( 'markdown.obsidian.personal.notation.html#main_of_notation',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.make_a_notation_note': ( 'markdown.obsidian.personal.notation.html#make_a_notation_note',
                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.make_notation_notes_from_double_asts': ( 'markdown.obsidian.personal.notation.html#make_notation_notes_from_double_asts',
                                                                                                                                                   'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_in_note': ( 'markdown.obsidian.personal.notation.html#notation_in_note',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_note_is_linked_in_see_also_section': ( 'markdown.obsidian.personal.notation.html#notation_note_is_linked_in_see_also_section',
                                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_notes_linked_in_see_also_section': ( 'markdown.obsidian.personal.notation.html#notation_notes_linked_in_see_also_section',
                                                                                                                                                        'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notation_str_in_a_standard_information_note': ( 'markdown.obsidian.personal.notation.html#notation_str_in_a_standard_information_note',
                                                                                                                                                          'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notations_and_main_notes': ( 'markdown.obsidian.personal.notation.html#notations_and_main_notes',
                                                                                                                                       'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.notations_to_add_in_index': ( 'markdown.obsidian.personal.notation.html#notations_to_add_in_index',
                                                                                                                                        'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.parse_notation_note': ( 'markdown.obsidian.personal.notation.html#parse_notation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.regex_from_latex': ( 'markdown.obsidian.personal.notation.html#regex_from_latex',
                                                                                                                               'trouver/markdown/obsidian/personal/notation.py'),
                                                             'trouver.markdown.obsidian.personal.notation.regex_from_notation_note': ( 'markdown.obsidian.personal.notation.html#regex_from_notation_note',
                                                                                                                                       'trouver/markdown/obsidian/personal/notation.py')},
            'trouver.markdown.obsidian.personal.note_processing': { 'trouver.markdown.obsidian.personal.note_processing.process_standard_information_note': ( 'markdown.obsidian.personal.note_processing.html#process_standard_information_note',
                                                                                                                                                              'trouver/markdown/obsidian/personal/note_processing.py'),
                                                                    'trouver.markdown.obsidian.personal.note_processing.remove_double_asterisks_in_markdown_file': ( 'markdown.obsidian.personal.note_processing.html#remove_double_asterisks_in_markdown_file',
                                                                                                                                                                     'trouver/markdown/obsidian/personal/note_processing.py')},
            'trouver.markdown.obsidian.personal.note_type': { 'trouver.markdown.obsidian.personal.note_type.NoteTypeError': ( 'markdown.obsidian.personal.note_type.html#notetypeerror',
                                                                                                                              'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.NoteTypeError.__init__': ( 'markdown.obsidian.personal.note_type.html#notetypeerror.__init__',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.PersonalNoteTypeEnum': ( 'markdown.obsidian.personal.note_type.html#personalnotetypeenum',
                                                                                                                                     'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.PersonalNoteTypeEnum.type_function': ( 'markdown.obsidian.personal.note_type.html#personalnotetypeenum.type_function',
                                                                                                                                                   'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_common_terms_note': ( 'markdown.obsidian.personal.note_type.html#_is_common_terms_note',
                                                                                                                                      'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_equation_note': ( 'markdown.obsidian.personal.note_type.html#_is_equation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_glossary_note': ( 'markdown.obsidian.personal.note_type.html#_is_glossary_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_index_note': ( 'markdown.obsidian.personal.note_type.html#_is_index_note',
                                                                                                                               'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_index_of_notation_note': ( 'markdown.obsidian.personal.note_type.html#_is_index_of_notation_note',
                                                                                                                                           'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_mathematician_note': ( 'markdown.obsidian.personal.note_type.html#_is_mathematician_note',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_module_information_note': ( 'markdown.obsidian.personal.note_type.html#_is_module_information_note',
                                                                                                                                            'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_notation_note': ( 'markdown.obsidian.personal.note_type.html#_is_notation_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_notes_note': ( 'markdown.obsidian.personal.note_type.html#_is_notes_note',
                                                                                                                               'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_premise_note': ( 'markdown.obsidian.personal.note_type.html#_is_premise_note',
                                                                                                                                 'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_reference_note': ( 'markdown.obsidian.personal.note_type.html#_is_reference_note',
                                                                                                                                   'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_standard_information_note': ( 'markdown.obsidian.personal.note_type.html#_is_standard_information_note',
                                                                                                                                              'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_tag_note': ( 'markdown.obsidian.personal.note_type.html#_is_tag_note',
                                                                                                                             'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type._is_template_note': ( 'markdown.obsidian.personal.note_type.html#_is_template_note',
                                                                                                                                  'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.assert_note_is_of_type': ( 'markdown.obsidian.personal.note_type.html#assert_note_is_of_type',
                                                                                                                                       'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.note_is_of_type': ( 'markdown.obsidian.personal.note_type.html#note_is_of_type',
                                                                                                                                'trouver/markdown/obsidian/personal/note_type.py'),
                                                              'trouver.markdown.obsidian.personal.note_type.type_of_note': ( 'markdown.obsidian.personal.note_type.html#type_of_note',
                                                                                                                             'trouver/markdown/obsidian/personal/note_type.py')},
            'trouver.markdown.obsidian.personal.notes': { 'trouver.markdown.obsidian.personal.notes.note_has_tag': ( 'markdown.obsidian.personal.notes.html#note_has_tag',
                                                                                                                     'trouver/markdown/obsidian/personal/notes.py'),
                                                          'trouver.markdown.obsidian.personal.notes.notes_linked_in_note': ( 'markdown.obsidian.personal.notes.html#notes_linked_in_note',
                                                                                                                             'trouver/markdown/obsidian/personal/notes.py'),
                                                          'trouver.markdown.obsidian.personal.notes.notes_linked_in_notes_linked_in_note': ( 'markdown.obsidian.personal.notes.html#notes_linked_in_notes_linked_in_note',
                                                                                                                                             'trouver/markdown/obsidian/personal/notes.py')},
            'trouver.markdown.obsidian.personal.reference': { 'trouver.markdown.obsidian.personal.reference._chapter_titles': ( 'markdown.obsidian.personal.reference.html#_chapter_titles',
                                                                                                                                'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._create_template_note_at': ( 'markdown.obsidian.personal.reference.html#_create_template_note_at',
                                                                                                                                         'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._files_in_reference_folder_via_glob': ( 'markdown.obsidian.personal.reference.html#_files_in_reference_folder_via_glob',
                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._files_in_reference_folder_via_index_notes': ( 'markdown.obsidian.personal.reference.html#_files_in_reference_folder_via_index_notes',
                                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_chapter_folders_and_indices': ( 'markdown.obsidian.personal.reference.html#_make_chapter_folders_and_indices',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_glossary_file': ( 'markdown.obsidian.personal.reference.html#_make_glossary_file',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_index_file': ( 'markdown.obsidian.personal.reference.html#_make_index_file',
                                                                                                                                 'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_notation_index_file': ( 'markdown.obsidian.personal.reference.html#_make_notation_index_file',
                                                                                                                                          'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_reference_file': ( 'markdown.obsidian.personal.reference.html#_make_reference_file',
                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_reference_folder': ( 'markdown.obsidian.personal.reference.html#_make_reference_folder',
                                                                                                                                       'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_single_chapter_folders': ( 'markdown.obsidian.personal.reference.html#_make_single_chapter_folders',
                                                                                                                                             'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_single_chapter_index': ( 'markdown.obsidian.personal.reference.html#_make_single_chapter_index',
                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_temp_folder': ( 'markdown.obsidian.personal.reference.html#_make_temp_folder',
                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._make_template_file': ( 'markdown.obsidian.personal.reference.html#_make_template_file',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._manifest_template_file': ( 'markdown.obsidian.personal.reference.html#_manifest_template_file',
                                                                                                                                        'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._modify_fast_link_edit_and_templates': ( 'markdown.obsidian.personal.reference.html#_modify_fast_link_edit_and_templates',
                                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference._obsidian_vault_plugin_configs_file': ( 'markdown.obsidian.personal.reference.html#_obsidian_vault_plugin_configs_file',
                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.copy_obsidian_vault_configs': ( 'markdown.obsidian.personal.reference.html#copy_obsidian_vault_configs',
                                                                                                                                            'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.copy_obsidian_vault_configs_with_nice_modifications': ( 'markdown.obsidian.personal.reference.html#copy_obsidian_vault_configs_with_nice_modifications',
                                                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.delete_reference_folder': ( 'markdown.obsidian.personal.reference.html#delete_reference_folder',
                                                                                                                                        'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.files_in_reference_folder': ( 'markdown.obsidian.personal.reference.html#files_in_reference_folder',
                                                                                                                                          'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_index_notes_from_index_note': ( 'markdown.obsidian.personal.reference.html#get_index_notes_from_index_note',
                                                                                                                                                'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_index_notes_from_subdirectory': ( 'markdown.obsidian.personal.reference.html#get_index_notes_from_subdirectory',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.get_obsidian_vault_plugin_configs': ( 'markdown.obsidian.personal.reference.html#get_obsidian_vault_plugin_configs',
                                                                                                                                                  'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.index_note_for_reference': ( 'markdown.obsidian.personal.reference.html#index_note_for_reference',
                                                                                                                                         'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.modify_obsidian_vault_plugin_configs': ( 'markdown.obsidian.personal.reference.html#modify_obsidian_vault_plugin_configs',
                                                                                                                                                     'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.reference_directory': ( 'markdown.obsidian.personal.reference.html#reference_directory',
                                                                                                                                    'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.reference_folders_in_vault': ( 'markdown.obsidian.personal.reference.html#reference_folders_in_vault',
                                                                                                                                           'trouver/markdown/obsidian/personal/reference.py'),
                                                              'trouver.markdown.obsidian.personal.reference.setup_folder_for_new_reference': ( 'markdown.obsidian.personal.reference.html#setup_folder_for_new_reference',
                                                                                                                                               'trouver/markdown/obsidian/personal/reference.py')},
            'trouver.markdown.obsidian.tags': { 'trouver.markdown.obsidian.tags.strip_auto_from_tag': ( 'markdown.obsidian.tags.html#strip_auto_from_tag',
                                                                                                        'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.tag_is_auto_tag': ( 'markdown.obsidian.tags.html#tag_is_auto_tag',
                                                                                                    'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.tag_without_hashtag': ( 'markdown.obsidian.tags.html#tag_without_hashtag',
                                                                                                        'trouver/markdown/obsidian/tags.py'),
                                                'trouver.markdown.obsidian.tags.to_tag_str': ( 'markdown.obsidian.tags.html#to_tag_str',
                                                                                               'trouver/markdown/obsidian/tags.py')},
            'trouver.markdown.obsidian.vault': { 'trouver.markdown.obsidian.vault.NoteDoesNotExistError': ( 'markdown.obsidian.vault.html#notedoesnotexisterror',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteDoesNotExistError.__init__': ( 'markdown.obsidian.vault.html#notedoesnotexisterror.__init__',
                                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteDoesNotExistError.from_note_name': ( 'markdown.obsidian.vault.html#notedoesnotexisterror.from_note_name',
                                                                                                                           'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError': ( 'markdown.obsidian.vault.html#notenotuniqueerror',
                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError.__init__': ( 'markdown.obsidian.vault.html#notenotuniqueerror.__init__',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NoteNotUniqueError.from_note_names': ( 'markdown.obsidian.vault.html#notenotuniqueerror.from_note_names',
                                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror',
                                                                                                                   'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError.__init__': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror.__init__',
                                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.NotePathIsNotIdentifiedError.from_note': ( 'markdown.obsidian.vault.html#notepathisnotidentifiederror.from_note',
                                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote': ( 'markdown.obsidian.vault.html#vaultnote',
                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.__init__': ( 'markdown.obsidian.vault.html#vaultnote.__init__',
                                                                                                         'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._add_single_entry_to_cache': ( 'markdown.obsidian.vault.html#vaultnote._add_single_entry_to_cache',
                                                                                                                           'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._check_if_cache_needs_to_update': ( 'markdown.obsidian.vault.html#vaultnote._check_if_cache_needs_to_update',
                                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._check_name_exists_and_unique_in_vault_cache': ( 'markdown.obsidian.vault.html#vaultnote._check_name_exists_and_unique_in_vault_cache',
                                                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._get_from_cache': ( 'markdown.obsidian.vault.html#vaultnote._get_from_cache',
                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._identify_rel_path': ( 'markdown.obsidian.vault.html#vaultnote._identify_rel_path',
                                                                                                                   'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote._remove_single_entry_from_cache': ( 'markdown.obsidian.vault.html#vaultnote._remove_single_entry_from_cache',
                                                                                                                                'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.clear_cache': ( 'markdown.obsidian.vault.html#vaultnote.clear_cache',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.create': ( 'markdown.obsidian.vault.html#vaultnote.create',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.delete': ( 'markdown.obsidian.vault.html#vaultnote.delete',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.directory': ( 'markdown.obsidian.vault.html#vaultnote.directory',
                                                                                                          'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.exists': ( 'markdown.obsidian.vault.html#vaultnote.exists',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.identify_rel_path': ( 'markdown.obsidian.vault.html#vaultnote.identify_rel_path',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.move_to': ( 'markdown.obsidian.vault.html#vaultnote.move_to',
                                                                                                        'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.move_to_folder': ( 'markdown.obsidian.vault.html#vaultnote.move_to_folder',
                                                                                                               'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.obsidian_identifier': ( 'markdown.obsidian.vault.html#vaultnote.obsidian_identifier',
                                                                                                                    'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.path': ( 'markdown.obsidian.vault.html#vaultnote.path',
                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.rel_path_identified': ( 'markdown.obsidian.vault.html#vaultnote.rel_path_identified',
                                                                                                                    'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.text': ( 'markdown.obsidian.vault.html#vaultnote.text',
                                                                                                     'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.unique_name': ( 'markdown.obsidian.vault.html#vaultnote.unique_name',
                                                                                                            'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.VaultNote.update_cache': ( 'markdown.obsidian.vault.html#vaultnote.update_cache',
                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.all_note_paths_by_name': ( 'markdown.obsidian.vault.html#all_note_paths_by_name',
                                                                                                             'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.all_paths_to_notes_in_vault': ( 'markdown.obsidian.vault.html#all_paths_to_notes_in_vault',
                                                                                                                  'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_name_from_path': ( 'markdown.obsidian.vault.html#note_name_from_path',
                                                                                                          'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_name_unique': ( 'markdown.obsidian.vault.html#note_name_unique',
                                                                                                       'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.note_path_by_name': ( 'markdown.obsidian.vault.html#note_path_by_name',
                                                                                                        'trouver/markdown/obsidian/vault.py'),
                                                 'trouver.markdown.obsidian.vault.path_to_obs_id': ( 'markdown.obsidian.vault.html#path_to_obs_id',
                                                                                                     'trouver/markdown/obsidian/vault.py')}}}
